c=======================================================================
c
c     Program ENDF2C
c     ==============
c     Convert ENDF Data to Standard FORTRAN, C and C++ Format.
c
c     Version 2014-1 Feb. 2014 * Initial version.
C             2014-2 Oct. 2014 * Changed from D to E exponential form
C                                to improve compatibility between
C                                computer languages.
C             2015-1 Jan. 2015 * General updates for release with
C                                PREPRO2015.
C                              * Changed ENDF data filenames from ENDF2C
C                                to ENDFB, to agree with PREPRO default
C                                definitions.
C                              * Added code name (to be compatible
C                                with PREPRO output), but NOT TIME (to
C                                keep this code as computer independent
C                                as possible).
C
c     Purpose
c     ==================================================================
c     This code is designed for,
c     1) ENDF Data in any ENDF format = ENDF-1 through ENDF-6.
c     2) On any type of computer = 32 or 64 bit system/compiler
c
c     This code tries to keep things as simple as possible
c     1) There are NO INPUT PARAMETERS.
c     2) It reads  an ENDF formatted file named ENDFB.IN
c     3) It writes an ENDF formatted file named ENDFB.OUT
c     4) It writes a report file named          ENDF2C.LST
c
c     Author's Message
c     ----------------
c     I consider insuring that ENDF data is in a standard, officially
c     approved format for FORTRAN, C and C++ is SO IMPORTANT this code
c     does only one thing - and only one thing - and it does it in the
c     simplest possible manner - efficiency is NOT a consideration -
c     ONLY accuracy and general utility of the ENDF data is considered.
c
c     Method
c     ------
c     Other codes that attempt to do the same thing - including codes
c     written be me decades ago - are very complicated, and therefore
c     ERROR PRONE because they try to deal with each and every variant
c     in which data can be coded in the ENDF format. Needless to say
c     this means that every time the ENDF formats and procedures change
c     these codes MUSE also be changed.
c
c     In contrast, ENDF2C uses my almost 50 years of experience dealing
c     with the ENDF format to realize that except for the comments at
c     the beginning for each evaluation (MF/MT=1/451), every line of
c     ENDF data is IDENTICAL - in every version of the ENDF format, from
c     the original ENDF to today's ENDF-6. So to translate ENDF data
c     into an official format I do not have to consider differences in
c     each section (MF/MT) of data.
c
c     Every line of ENDF is divided into 6 fields, each 11 columns wide.
c     Each of the 6 fields is either, blank, integer or floating point.
c     Floating point fields ALL include a decimal point (.). So that ALL
c     this code does is convert every floating point field to standard
c     format.
c
c     In order to insure that this PRESERVES the accuracy of the data
c     this is done by reading and writing each ENDF line as characters.
c     Blank and integer fields are copied exactly as read. ALL floating
c     point number that are read are converted internally from character
c     to floating point - they are then converted back into characters
c     in a standard, officially approved format, for output.
c
c     As a last step to insure the accuracy of results the characters
c     to be output are again converted from characters to floating
c     point, and the numerical value that is output is compared to the
c     numerical value originally read, and if there is ANY DIFFERENCE
c     the characters strings read and written are listed in the output:
c     the characters strings read and written as well as the difference
c     is listed in the output report (ENDF2C.LST) and on the screen.
c
c     Running Time
c     ------------
c     It takes only seconds to translate an ENDF formatted evaluation,
c     so running time need not be a consideration. Concentrate on
c     keeping it simple and reliable - that should be your focus.
c
c     Documentation
c     -------------
c     ALL of my codes that process ENDF data and change it in ANY WAY
c     document what they have done by adding comment lines at the end
c     of the comment section (MF/MT=1/451) of each evaluation. This
c     allows data users to determine the pedigree of the data they are
c     using, by reading these comments. This code documents what is has
c     done by adding the following 2 comment lines.
c
c ***************** Program ENDF2C (Version 2015-1) ***************
c      Convert ENDF Data to Standard FORTRAN, C and C++ Format
c
c     WARNING - This documentation is IMPORTANT to data users and it
c     should not be deleted.
c
c     Written by
c     ------------------------------------
c     Dermott E. Cullen
c     University of California (retired)
c-----Present Home Address----------------------------------------------
c     Dermott E. Cullen
c     1466 Hudson Way
c     Livermore, CA 94550
c     U.S.A.
c     Telephone  925-443-1911
c     E. Mail    RedCullen1@Comcast.net
c     Website    http://home.comcast.net/~redcullen1
c
c=======================================================================
      implicit real*8 (a-h,o-z)
      implicit integer*4 (i-n)
      save
      character*8 codename
      character*1 LINE66
      common/endf1/LINE66(11,6)
      common/endf2/MAT,MF,MT,NOSEQ,IVERSE
      common/namecode/codename
c-----------------------------------------------------------------------
c
c     Define Code name for output report
c
c-----------------------------------------------------------------------
      codename = 'ENDF2C  '
c-----------------------------------------------------------------------
c
c     Define I/O Files
c
c-----------------------------------------------------------------------
      open( 3,file='ENDF2C.LST')
      open(10,file='ENDFB.IN')
      open(11,file='ENDFB.OUT')
c-----------------------------------------------------------------------
c
c     Identify Code.
c
c-----------------------------------------------------------------------
      write(*,1020)
      write(3,1020)
 1020 format(//
     1 ' ',13('*'),'**** Program ENDF2C (Version 2015-1) ***',
     2 12('*')/5(' '),
     3 ' Convert ENDF Data to Standard FORTRAN, C and C++ Format',
     4 5(' ')/1x,71('-')/' ENDF Tape Label (TPID)'/1x,71('-'))
c-----Initialize previous MAT read and ENDF format version #
      MATLAST = -99999
      IVERSE  = 0
      NOSEQ   = 0
c-----------------------------------------------------------------------
c
c     copy TPID - and identify it in output listing
c
c-----------------------------------------------------------------------
      read (10,1000) LINE66,MAT,MF,MT
      write( *,1000) LINE66,MAT,MF,MT,NOSEQ
      write( 3,1000) LINE66,MAT,MF,MT,NOSEQ
 1000 format(66A1,i4,i2,i3,i5)
c-----------------------------------------------------------------------
c
c     Check for expected TPID format = file may not have TPID.
c
c-----------------------------------------------------------------------
      if(MF.eq.0.and.MT.eq.0) then
      NOSEQ = 0
      write(11,1000) LINE66,MAT,MF,MT,NOSEQ               ! Assume TPID
      write( *,1040)
      write( 3,1040)
 1040 format(1x,71('-')/' Input       Output         Fractional Diff.',
     5 '    MAT MF  MT  Seq# Format'/
     6       1x,71('-'))
      else
      write( *,1050)                                      ! NOT TPID
      write( 3,1050)
 1050 format(1x,71('-')/' WARNING - This DOES NOT look like a TPID.',
     1       ' Expect MF = MT = 0 on TPID.'/
     2       '           Will Assume file has NO TPID',
     3       ' and TRY to Translate this line.'/
     4 1x,71('-')/' Input       Output         Fractional Diff.',
     5 '    MAT MF  MT  Seq# Format'/
     6       1x,71('-'))
      go to 20                                            ! Translate
      endif
c-----------------------------------------------------------------------
c
c     Loop over ALL ENDF lines to TEND line
c
c-----------------------------------------------------------------------
   10 read (10,1000,end=500) LINE66,MAT,MF,MT
c-----Initialize ENDF Format vesion # for each new MAT
   20 if(MAT.ne.MATLAST) then
      MATLAST = MAT
      IVERSE  = 0
      NOSEQ   = 0
      endif
c-----------------------------------------------------------------------
c
c     special treatment for SEND/FEND/MEND/TEND
c
c-----------------------------------------------------------------------
      if(MT.le.0) then               ! SEND/FEND/MEND/TEND?
      if(MF.gt.0) then
      NOSEQ = 99999                  ! SEND NOSEQ = 99999
      else
      NOSEQ = 0                      ! FEND/MEND/TEND NOSEQ = 0
      endif
      write(11,1060) MAT,MF,MT,NOSEQ ! Output with blank fields
 1060 format(66X,i4,i2,i3,i5)
      NOSEQ = 0                      ! Initialze before next line.
      if(MAT.lt.0) then
      go to 600                      ! TEND = End of ENDF data
      else
      go to 10                       ! Otherwise continue reading
      endif
      endif
c-----------------------------------------------------------------------
c
c     special treatment MF/MT=1/451 = Add Documentation
c
c-----------------------------------------------------------------------
      if(MF.eq.1.and.MT.eq.451) then
      call FILE1
      go to 10
      endif
c-----------------------------------------------------------------------
c
c     No other special cases - translate 6 data fields to standard form
c
c-----------------------------------------------------------------------
      NOSEQ = NXTSEQ(NOSEQ)
      call STANDARD(6)
c-----Output standard format
      write(11,1000) LINE66,MAT,MF,MT,NOSEQ
      go to 10
c-----------------------------------------------------------------------
c
c     End of ENDF input
c
c-----------------------------------------------------------------------
c-----End of File BEFORE TEND
  500 write( *,1070)
      write( 3,1070)
 1070 format(1x,71('-')/
     1 ' ERROR - ENDF file does not end with TEND Line.'/1x,71('-'))
      call enderror
c-----TEND line read
  600 call endit
      end
      Subroutine FILE1
C=======================================================================
c
c     Add documentation to MF/MT=1/451
c
c     ENDF-4 Format: 2-nd line N1 > 0, N2 = 0
c         -5       : 3-rd line N1 = 0, N2 = 0
c         -6       : 4-th line N1 > 0, N2 > 0
c
C=======================================================================
      implicit real*8 (a-h,o-z)
      implicit integer*4 (i-n)
      save
      character*1 LINE66,LINE44(44),LINE11(11),MTDICT(44)
      common/endf1/LINE66(11,6)
      common/endf2/MAT,MF,MT,NOSEQ,IVERSE
      equivalence (LINE66(1,1),LINE44(1)),(LINE66(1,6),LINE11)
c-----Initialize ENDF format version
      IVERSE = 0
c
c     first line already read - translate C1 and C2 and output
c
      NOSEQ = NXTSEQ(NOSEQ)
      call STANDARD(2)
      write(11,1000) LINE66,MAT,MF,MT,NOSEQ
c-----------------------------------------------------------------------
c
c     Loop over UP TO next 3 lines = fewer for earlier ENDF versions.
c
c-----------------------------------------------------------------------
      do 40 iline=1,3
      read (10,1000) LINE66,MAT,MF,MT
 1000 format(66A1,i4,i2,i3,i5)
c-----Convert C1 and C2 fields
      NOSEQ = NXTSEQ(NOSEQ)
      call STANDARD(2)
c-----Define ENDF format version based on N1 and N2
      call in9(XN1,LINE66(1,5))
      call in9(XN2,LINE66(1,6))
      N1IN = XN1
      N2IN = XN2
c-----Based on which card read
      go to (10,20,30),iline
   10 N1A = N1IN
      N2A = N2IN
      if(N1A.le.0.or.N2A.ne.0) go to 40
      IVERSE = 4                              ! IV Format
      NWD    = N1A
      go to 50
   20 N1B = N1IN
      N2B = N2IN
      if(N2A.gt.0) go to 40
      IVERSE = 5                              ! V  Format
      NWD    = N1B
      go to 50
   30 N1C = N1IN
      N2C = N2IN
      IVERSE = 6                              ! VI Format
      NWD    = N1C
      go to 50
   40 write(11,1000) LINE66,MAT,MF,MT,NOSEQ   ! Copy line = no change.
c-----Output updated comment count
   50 NWDOUT = NWD + 2
      write(11,1010) LINE44,NWDOUT,LINE11,MAT,MF,MT,NOSEQ ! New count
 1010 format(44A1,i11,11A1,i4,i2,i3,i5)
c-----------------------------------------------------------------------
c
c     Copy comments = insure first 22 coilumns are blank.
c
c-----------------------------------------------------------------------
      do iwd = 1,NWD
      read (10,1000) LINE66,MAT,MF,MT
      NOSEQ = NXTSEQ(NOSEQ)
      write(11,1000) LINE66,MAT,MF,MT,NOSEQ
      enddo
c-----------------------------------------------------------------------
c
c     Add comments + new sequence numbers
c
c-----------------------------------------------------------------------
      NOSEQ = NXTSEQ(NOSEQ)
      write(11,1030) MAT,MF,MT,NOSEQ
      NOSEQ = NXTSEQ(NOSEQ)
      write(11,1040) MAT,MF,MT,NOSEQ
 1030 format(
     1 ' ',13('*'),'**** Program ENDF2C (Version 2015-1) ***',
     2 12('*'),i4,i2,i3,i5)
 1040 format(5(' '),
     1 'Convert ENDF Data to Standard FORTRAN, C and C++ Format',
     2 6(' '),i4,i2,i3,i5)
c-----------------------------------------------------------------------
c
c     Copy to SEND
c
c-----------------------------------------------------------------------
   60 read (10,1020) MTDICT,MAT,MF,MT
 1020 format(22x,44A1,i4,i2,i3,i5)
      if(MT.gt.0) then
      NOSEQ = NXTSEQ(NOSEQ)
      write(11,1020) MTDICT,MAT,MF,MT,NOSEQ
      go to 60
      endif
      NOSEQ = 99999                     ! SEND line
      write(11,1050) MAT,MF,MT,NOSEQ
 1050 format(66x,i4,i2,i3,i5)
      NOSEQ = 0                         ! Initialize for next line
      return
      end
      Subroutine STANDARD(NFIELD)
C=======================================================================
c
c     Scan ALL other lines
c
c     NFIELD = # of fields to scan = 2 or 6
c
C=======================================================================
      implicit real*8 (a-h,o-z)
      implicit integer*4 (i-n)
      save
      character*1 LINE66,digits(0:9)
      data digits/'0','1','2','3','4','5','6','7','8','9'/
      character*1 LINESAVE(11)
      common/endf1/LINE66(11,6)
      common/endf2/MAT,MF,MT,NOSEQ,IVERSE
c-----------------------------------------------------------------------
c
c     Scan fields for . No other test is necessary
c
c-----------------------------------------------------------------------
      do 20 i = 1,NFIELD
      do k = 1,11
      if(LINE66(k,i).eq.'.') go to 10
      enddo
c-----no . = nothing to change
      go to 20
c-----save original form
   10 do kk=1,11
      LINESAVE(kk) = LINE66(kk,i)
      enddo
c-----translate
      call IN9 (zin,LINE66(1,i))
c-----Try for 10 digit output - switch to 9 if negative.
      call OUT10(zin,LINE66(1,i))
c-----------------------------------------------------------------------
c
c     Compare original and final = Print if significantly different
c
c-----------------------------------------------------------------------
      call IN9 (zix,LINE66(1,i))
      if(DABS(zin).le.0.0d+0) then   ! Avoid divding by zero.
      diff = (zin - zix)
      else
      diff = (zin - zix)/zin
      endif
      if(dabs(diff).gt.1.0d-10) then
      write(*,6000) LINESAVE,(LINE66(kk,i),kk=1,11),diff,
     1 MAT,MF,MT,NOSEQ,IVERSE
      write(3,6000) LINESAVE,(LINE66(kk,i),kk=1,11),diff,
     1 MAT,MF,MT,NOSEQ,IVERSE
 6000 format(1x,11a1,1x,11a1,2x,1pd20.12,i5,i3,i4,i6,i5)
      endif
c-----end of field loop
   20 continue
      return
      end
      Function NXTSEQ(NOSEQ)
C=======================================================================
C
C     DEFINE NEXT SEQUENCE NUMBER FOR ENDF/B OUTPUT. ALLOW FOR
C     MORE THAN 100000 LINES PER EVALUATION BY RESETTING NUMBER
C     TO 1 EVERY TIME 100000 IS REACHED.
C
C=======================================================================
      implicit real*8 (a-h,o-z)
      implicit integer*4 (i-n)
      save
      NN=NOSEQ+1
      IF(NN.ge.100000) NN=1
      NXTSEQ=NN
      RETURN
      END
      SUBROUTINE OUT9(ZIN,FIELD)
C=======================================================================
C
C     PURPOSE
C     =======
C     FORMAT NUMBER FOR OUTPUT TO INCLUDE AS MANY DIGITS OF
C     ACCURACY AS POSSIBLE.
C     12/18/2012 - EXTENDED FOR 3 DIGIT EXPONENT
C                  No test for 4 or more digit exponent
C     10/25/2014 - Changed from D to E exponential form to improve
C                  compatibility between computer languages
C
C     040923 - CHANGED ZLOW FROM 1.0D-4 TO 1.0D-3
C              NEAR 1.0D-3 PRECISION GOES FROM 999,999 TO 1,000,000
C              FOR SMOOTHEST TRANSITION FROM 6 TO 7 DIGITS
C
C     ARGUMENTS
C     =========
C     Z        = FLOATING POINT NUMBER OF BE OUTPUT (INPUT)
C     FIELD    = 11A1 CHARACTERS TO OUTPUT          (OUTPUT)
C
C     METHOD
C     ======
C     COLUMNS            12345678901     ACCURACY
C     -------------------------------------------
C     0 TO 10^-9          1.2345E-12     5 DIGITS
C     10^-9 TO 10^-3      1.23456E-8     6 DIGITS
C     10^-3 TO 10^-2      .001234567     7 DIGITS
C     10^-2 TO 10^-1      .012345678     8 DIGITS
C     10^-1 TO 1          .123456789     9 DIGITS
C     1 TO 10^9           12345.6789     9 DIGITS
C     10^9 TO 10^10       1.23456E+9     6 DIGITS
C     10^10 >             1.2345E+12     5 DIGITS
C
C     OUTPUT WILL BE IN 11 COLUMN FORMAT
C
C     WARNING - THIS IS NOT A GENERAL ROUNDING ROUTINE WHICH WILL WORK
C               FOR ROUNDING TO ANY NUMBER OF DIGITS - IT WILL ONLY
C               WORK PROPERLY FOR THE RANGES INDICATED ABOVE, FOR
C               11 COLUMN OUTPUT.
C
C=======================================================================
      implicit real*8 (a-h,o-z)
      implicit integer*4 (i-n)
      save
c-----INTEGER*8 for long integers
      integer*8 INMANT,INMAN9,INMAN8,INMAN7,INMAN6,INMAN5,INMAN4
      CHARACTER*1 FIELD,DIGITS,ZEROH
C-----12/18/2012 - EXTENDED FOR 3 DIGIT EXPONENT
      DIMENSION DIGITS(0:9),FIELD(11),ZEROH(11),
     1 TENS(-99:99),ROUNDER(-99:99)
      DATA DIGITS/
     1 '0','1','2','3','4','5','6','7','8','9'/
C-----RETURN FOR = 0
      DATA ZEROH/
     1 ' ','0','.','0',' ',' ',' ',' ',' ',' ',' '/
C-----LOWER TRANSITION POINT FROM 7 TO 9 DIGIT OUTPUT
      DATA ZLOW/1.0D-03/
C-----UPPER TRANSITION POINT FROM 9 TO 7 DIGIT OUTPUT
      DATA ZHIGH/1.0D+09/
      DATA TENTH/1.0D-01/
c-----------------------------------------------------------------------
c
c     ON FIRST CALL INITIALIZE POWERS OF 10
c
c-----------------------------------------------------------------------
      DATA IPASS/0/
      IF(IPASS.NE.0) GO TO 50
      IPASS=1
c-----INTEGER*8 for long integers
      INMAN4  = 10000
      INMAN5  = 10*INMAN4
      INMAN6  = 10*INMAN5
      INMAN7  = 10*INMAN6
      INMAN8  = 10*INMAN7
      INMAN9  = 10*INMAN8
      TENS(0)=1.0D+00
C-----12/18/2012 - EXTENDED FOR 3 DIGIT EXPONENT
C-----11/22/2013 - Restricted to 2 Digit Exponent
      DO 10 I=1,99
      TENS( I)=TENS(I-1)*10.0D+00
      TENS(-I)=TENS(1-I)/10.0D+00
   10 ROUNDER(I)  = 5.001D-05
      DO 20 I=-99,-10
   20 ROUNDER(I)  = 5.001D-05
      DO 30 I=-9,-4
   30 ROUNDER(I)  = 5.001D-06
      ROUNDER(-3) = 5.001D-07
      ROUNDER(-2) = 5.001D-08
      ROUNDER(-1) = 5.001D-09
      ROUNDER( 0) = 5.001D-09
      DO 40 I=1,8
   40 ROUNDER(I)  = 5.001D-09
      ROUNDER(9)  = 5.001D-06
c-----------------------------------------------------------------------
C
C     IMMEDIATELY RETURN 0.00000+00.
C
c-----------------------------------------------------------------------
C-----02/14/04 - ADDED, JUST IN CASE
   50 IF(DABS(ZIN).le.0.0D+0) GO TO 60
      IF(DABS(ZIN).lt.1.0D-99) GO TO 60    ! Only 2 digit exponents
      IF(DABS(ZIN).gt.1.0D+99) GO TO 60
c----- ZIN = 0 handled above
      IF(ZIN.lt.0.0d+0) go to 80
      go to 90
c-----Return 0
   60 DO 70 I=1,11
   70 FIELD(I)=ZEROH(I)
      RETURN
c-----------------------------------------------------------------------
C
C     DEFINE SIGN OF MANTISSA AND ABSOLUTE MANTISSA
C
c-----------------------------------------------------------------------
C-----NEGATIVE.
   80 FIELD(1)='-'
      Z=-ZIN
      GO TO 100
C-----POSITIVE.
   90 FIELD(1)=' '
      Z=ZIN
c-----------------------------------------------------------------------
c
c     DEFINE EXPONENT AND NORMALIZED MANTISSA
c
c-----------------------------------------------------------------------
  100 IEXP=DLOG10(Z)
      IF(Z.LT.1.0D+00) IEXP = IEXP - 1
c-----11/22/2013 - Decide here on F or E Output
c-----11/26/2013 - Restrict to 2 digit exponent - may change by 1 or 2
      if(IABS(IEXP).gt.97) go to 60
      if(ZIN.lt.0.0D+0) then
      ZN=Z*TENS(-IEXP) + ROUNDER(IEXP)       ! Standard
      else
      if(Z.LE.ZLOW.OR.Z.GE.ZHIGH) then
      ZN=Z*TENS(-IEXP) + TENTH*ROUNDER(IEXP) ! Extra Digit
      else
      ZN=Z*TENS(-IEXP) + ROUNDER(IEXP)       ! Standard
      endif
      endif
      IF(ZN.eq.1.0D+00) go to 120            ! Test rounding underflow
      IF(ZN.gt.1.0D+00) go to 110            ! Test rounding underflow
      IEXP=IEXP-1         ! MUST be < 1
      GO TO 120
  110 IF(ZN.lt.10.0D+00) go to 120           ! Test rounding overflow
      IEXP=IEXP+1         ! MUST be >= 10
      ZN=ZN/10.0D+00
  120 Z = ZN*TENS(IEXP)
c-----------------------------------------------------------------------
C
C     SELECT F OR E FORMAT
C
c-----------------------------------------------------------------------
      IF(Z.LE.ZLOW.OR.Z.GE.ZHIGH) GO TO 150
c-----------------------------------------------------------------------
C
C     F FORMAT
C
C     12345678901
C      X.XXXXXXXX = 9 DIGITS
C      .001234567
C      123456789.
C
c-----------------------------------------------------------------------
C-----DEFINE 6 TO 9 DIGIT MANTISSA WITH ROUNDING
      IPOWER=8-IEXP
      IF(IEXP.LT.0) IPOWER=8
      INMANT=Z*TENS(IPOWER)
C-----CHECK FOR OVERFLOW DUE TO ROUNDING
      if(INMANT.lt.INMAN9) go to 130
      INMANT=INMAN8
      IEXP=IEXP+1
C-----DECIMAL POINT.
  130 IDOT=3+IEXP
      IF(IDOT.LE.2) THEN
C----- IF < 1, MOVE DECIMAL POINT TO COLUMN 2 AND ADD A DIGIT
      IDOT=2
      INMANT=Z*10.0D+00*TENS(IPOWER)
      ENDIF
      FIELD(IDOT)='.'
C-----MANTISSA - LAST DIGIT TO FIRST.
      II=11
      DO 140 I=2,11
      IF(II.EQ.IDOT) GO TO 140
      I2=INMANT/10
      I3=INMANT-10*I2
      FIELD(II)=DIGITS(I3)
      INMANT=I2
  140 II=II-1
      RETURN
c-----------------------------------------------------------------------
C
C     E FORMAT
C
C     12345678901
C      X.XXXXE+NN = 5 DIGITS
C      X.XXXXXE+N = 6 DIGITS
C
C     11/22/2013 - If not negative, use first column
C     12345678901
C     X.XXXXXE+NN = 6 DIGITS
C     X.XXXXXXE+N = 7 DIGITS
c
c-----------------------------------------------------------------------
C-----Negative?
  150 IF(ZIN.lt.0.0d+0) go to 170
c
c     POsitive. Use first column - Decimal point is always in column 2
c
      FIELD(2)='.'
      KDOT    = 2
      ISTART  = 1
      IF(IABS(IEXP).GE.10) GO TO 160
      ID=8                               ! 1 Digit exponent
      INMANT=(1.0D+06)*ZN
      IF(INMANT.lt.INMAN7) go to 190
      INMANT=INMAN6
      IEXP=IEXP+1
      IF(IABS(IEXP).LT.10) GO TO 190
  160 ID=7                               ! 2 Digit exponent
      INMANT=(1.0D+05)*ZN
C-----CHECK FOR OVERFLOW DUE TO ROUNDING
      IF(INMANT.lt.INMAN6) go to 190
      INMANT=INMAN5
      IEXP=IEXP+1
c
c     Negative Number - Cannot use first column
c
C-----DECIMAL POINT IS ALWAYS IN COLUMN 3
  170 FIELD(3)='.'
      KDOT    = 3
      ISTART  = 2
      IF(IABS(IEXP).GE.10) GO TO 180
      ID=8                                 ! 1 Digit Exponent
      INMANT=(1.0D+05)*ZN
      IF(INMANT.lt.INMAN6) go to 190
      INMANT=INMAN5
      IEXP=IEXP+1
      IF(IABS(IEXP).LT.10) GO TO 190
  180 ID=7                                 ! 2 Digit Exponent
      INMANT=(1.0D+04)*ZN
      IF(INMANT.lt.INMAN5) go to 190
      INMANT=INMAN4
      IEXP=IEXP+1
C
C     DEFINE MANTISSA
C
  190 IEXPS=ID+1
      II=ID
      DO 200 I=ISTART,ID
      IF(II.EQ.KDOT) GO TO 200
      I2=INMANT/10
      I3=INMANT-10*I2
      FIELD(II)=DIGITS(I3)
      INMANT=I2
  200 II=II-1
C
C     E
C
      FIELD(IEXPS) = 'E'
      IEXPS = IEXPS + 1
C
C     SIGN OF EXPONENT
C
      IF(IEXP.ge.0) go to 210
      IEXP=-IEXP
      FIELD(IEXPS)='-'
      GO TO 220
  210 FIELD(IEXPS)='+'
C
C     EXPONENT
C
  220 IF(IEXP.lt.10) go to 230
      KEXP=IEXP/10
      FIELD(10)=DIGITS(KEXP)
      IEXP=MOD(IEXP,10)
  230 FIELD(11)=DIGITS(IEXP)
c
c     If using column 1 but mantissa ends in 0, move mantissa right.
c
      if(KDOT.eq.2.and.FIELD(ID).eq.'0') then
      do k=ID-1,1,-1
      FIELD(k+1)=FIELD(k)
      enddo
      FIELD(1) = ' '
      endif
      RETURN
      END
      SUBROUTINE OUT10(ZIN,FIELD)
C=======================================================================
C
C     PURPOSE
C     =======
C     FORMAT NUMBER FOR OUTPUT TO INCLUDE AS MANY DIGITS OF
C     ACCURACY AS POSSIBLE.
C     12/18/2012 - EXTENDED FOR 3 DIGIT EXPONENT
C                  No test for 4 or more digit exponent
C     10/25/2014 - Changed from D to E exponential form to improve
C                  compatibility between computer languages
C
C     040923 - CHANGED ZLOW FROM 1.0D-4 TO 1.0D-3
C              NEAR 1.0D-3 PRECISION GOES FROM 999,999 TO 1,000,000
C              FOR SMOOTHEST TRANSITION FROM 6 TO 7 DIGITS
C
C     ARGUMENTS
C     =========
C     Z        = FLOATING POINT NUMBER OF BE OUTPUT (INPUT)
C     FIELD    = 11A1 CHARACTERS TO OUTPUT          (OUTPUT)
C
C     METHOD
C     ======
C     COLUMNS            12345678901     ACCURACY
C     -------------------------------------------
C     0 TO 10^-9         1.23456E-12     6 DIGITS
C     10^-9 TO 10^-3     1.234567E-8     7 DIGITS
C     10^-3 TO 10^-2     .0012345678     8 DIGITS
C     10^-2 TO 10^-1     .0123456789     9 DIGITS
C     10^-1 TO 1         .1234567891    10 DIGITS
C     1 TO 10^9          12345.67891    10 DIGITS
C     10^9 TO 10^10      1.234567E+9     7 DIGITS
C     10^10 >            1.23456E+12     6 DIGITS
C
C     OUTPUT WILL BE IN 11 COLUMN FORMAT
C
C     WARNING - THIS IS NOT A GENERAL ROUNDING ROUTINE WHICH WILL WORK
C               FOR ROUNDING TO ANY NUMBER OF DIGITS - IT WILL ONLY
C               WORK PROPERLY FOR THE RANGES INDICATED ABOVE, FOR
C               11 COLUMN OUTPUT.
C
C=======================================================================
      implicit real*8 (a-h,o-z)
      implicit integer*4 (i-n)
      save
c-----INTEGER*8 for long integers
      integer*8 INMANT,INMAN10,INMAN9,INMAN7,INMAN6,INMAN5,INMAN4
      CHARACTER*1 FIELD,DIGITS,ZEROH
C-----12/18/2012 - EXTENDED FOR 3 DIGIT EXPONENT
      DIMENSION DIGITS(0:9),FIELD(11),ZEROH(11),
     1 TENS(-99:99),ROUNDER(-99:99)
      DATA DIGITS/
     1 '0','1','2','3','4','5','6','7','8','9'/
C-----RETURN FOR = 0
      DATA ZEROH/
     1 ' ','0','.','0',' ',' ',' ',' ',' ',' ',' '/
C-----LOWER TRANSITION POINT FROM 7 TO 9 DIGIT OUTPUT
      DATA ZLOW/1.0D-03/
C-----UPPER TRANSITION POINT FROM 9 TO 7 DIGIT OUTPUT
      DATA ZHIGH/1.0D+09/
      DATA TENTH/1.0D-01/
c-----------------------------------------------------------------------
c
c     ON FIRST CALL INITIALIZE POWERS OF 10
c
c-----------------------------------------------------------------------
      DATA IPASS/0/
      IF(IPASS.NE.0) GO TO 50
      IPASS=1
c-----INTEGER*8 for long integers
      INMAN4  = 10000
      INMAN5  = 10*INMAN4
      INMAN6  = 10*INMAN5
      INMAN7  = 10*INMAN6
      INMAN9  = 100*INMAN7
      INMAN10 = 10*INMAN9  ! WARNING - this exceeds 32 bit integer
      TENS(0)=1.0D+00
C-----12/18/2012 - EXTENDED FOR 3 DIGIT EXPONENT
C-----11/22/2013 - Restricted to 2 Digit Exponent
      DO 10 I=1,99
      TENS( I)=TENS(I-1)*10.0D+00
      TENS(-I)=TENS(1-I)/10.0D+00
   10 ROUNDER(I)  = 5.001D-05
      DO 20 I=-99,-10
   20 ROUNDER(I)  = 5.001D-05
      DO 30 I=-9,-4
   30 ROUNDER(I)  = 5.001D-06
      ROUNDER(-3) = 5.001D-07
      ROUNDER(-2) = 5.001D-08
      ROUNDER(-1) = 5.001D-09
      ROUNDER( 0) = 5.001D-09
      DO 40 I=1,8
   40 ROUNDER(I)  = 5.001D-09
      ROUNDER(9)  = 5.001D-06
c-----------------------------------------------------------------------
C
C     IMMEDIATELY RETURN 0.00000+00.
C
c-----------------------------------------------------------------------
C-----02/14/04 - ADDED, JUST IN CASE
   50 IF(DABS(ZIN).le.0.0D+0) GO TO 60
      IF(DABS(ZIN).lt.1.0D-99) GO TO 60    ! Only 2 digit expoents
      IF(DABS(ZIN).gt.1.0D+99) GO TO 60
c----- ZIN = 0 handled above
      IF(ZIN.lt.0.0d+0) go to 80
      go to 90
   60 DO 70 I=1,11
   70 FIELD(I)=ZEROH(I)
      RETURN
c-----------------------------------------------------------------------
C
C     DEFINE SIGN OF MANTISSA AND ABSOLUTE MANTISSA
C
c-----------------------------------------------------------------------
c-----< 0 = use OUT9
   80 CALL OUT9(ZIN,FIELD)
      RETURN
C-----POSITIVE.
   90 FIELD(1)=' '
      Z=ZIN
c-----------------------------------------------------------------------
c
c     DEFINE EXPONENT AND NORMALIZED MANTISSA
c
c-----------------------------------------------------------------------
      IEXP=DLOG10(Z)
c-----11/26/2013 - Restrict to 2 digit exponent - may change by 1 or 2
      if(IABS(IEXP).gt.97) go to 60
      IF(Z.LT.1.0D+00) IEXP = IEXP - 1
c-----11/22/2013 - Decide here on F or E Output
      if(ZIN.lt.0.0D+0) then
      ZN=Z*TENS(-IEXP) + ROUNDER(IEXP)       ! Standard
      else
      if(Z.LE.ZLOW.OR.Z.GE.ZHIGH) then
      ZN=Z*TENS(-IEXP) + TENTH*ROUNDER(IEXP) ! Extra Digit
      else
      ZN=Z*TENS(-IEXP) + TENTH*ROUNDER(IEXP)       ! Standard
      endif
      endif
      IF(ZN.eq.1.0D+00) go to 110
      IF(ZN.gt.1.0D+00) go to 100
      IEXP=IEXP-1               ! MUST be < 1
      go to 110
  100 IF(ZN.lt.10.0D+00) go to 110
      IEXP=IEXP+1               ! MUST be >= 10
      ZN=ZN/10.0D+00
  110 Z = ZN*TENS(IEXP)
c-----------------------------------------------------------------------
C
C     SELECT F OR E FORMAT
C
c-----------------------------------------------------------------------
      IF(Z.LE.ZLOW.OR.Z.GE.ZHIGH) GO TO 140
c-----------------------------------------------------------------------
C
C     F FORMAT
C
C     12345678901
C     X.XXXXXXXXX = 10 DIGITS
C     .0012345678
C     1234567891.
C
c-----------------------------------------------------------------------
C-----DEFINE 7 TO 10 DIGIT MANTISSA WITH ROUNDING
      IPOWER=9-IEXP
      IF(IEXP.LT.0) IPOWER=9
      INMANT=Z*TENS(IPOWER)
C-----CHECK FOR OVERFLOW DUE TO ROUNDING
      IF(INMANT.lt.INMAN10) go to 120
      INMANT=INMAN9
      IEXP=IEXP+1
C-----DECIMAL POINT.
  120 IDOT=2+IEXP
      IF(IDOT.LE.1) THEN
C----- IF < 1, MOVE DECIMAL POINT TO COLUMN 1 AND ADD A DIGIT
      IDOT=1
      INMANT=Z*10.0D+00*TENS(IPOWER)
      ENDIF
      FIELD(IDOT)='.'
C-----MANTISSA - LAST DIGIT TO FIRST.
      II=11
      DO 130 I=1,11
      IF(II.EQ.IDOT) GO TO 130
      I2=INMANT/10
      I3=INMANT-10*I2
      FIELD(II)=DIGITS(I3)
      INMANT=I2
  130 II=II-1
c
c     If 10 digit ends in 0, shift right = leave column 1 blank
c
      if(FIELD(1).ne.'-'.and.FIELD(11).eq.'0') then
      do k=10,1,-1
      FIELD(k+1)=FIELD(k)
      enddo
      FIELD(1) = ' '
      endif
      RETURN
c-----------------------------------------------------------------------
C
C     E FORMAT
C
C     12345678901
C      X.XXXXE+NN = 5 DIGITS
C      X.XXXXXE+N = 6 DIGITS
c
c     11/22/2013 - If first column is blank use it.
C
C     12345678901
C     X.XXXXXE+NN = 6 DIGITS
C     X.XXXXXXE+N = 7 DIGITS
C
C==============================================================
C-----Negative?
  140 IF(ZIN.lt.0.0d+0) go to 160
c
c     POsitive. Use first column - Decimal point is always in column 2
c
      FIELD(2)='.'
      KDOT    = 2
      ISTART  = 1
      IF(IABS(IEXP).GE.10) GO TO 150
      ID=8                               ! 1 Digit exponent
      INMANT=(1.0D+06)*ZN
      IF(INMANT.lt.INMAN7) go to 180
      INMANT=INMAN6
      IEXP=IEXP+1
      IF(IABS(IEXP).LT.10) GO TO 180
  150 ID=7                               ! 2 Digit exponent
      INMANT=(1.0D+05)*ZN
C-----CHECK FOR OVERFLOW DUE TO ROUNDING
      IF(INMANT.lt.INMAN6) go to 180
      INMANT=INMAN5
      IEXP=IEXP+1
c
c     Negative Number - Cannot use first column
c
C-----DECIMAL POINT IS ALWAYS IN COLUMN 3
  160 FIELD(3)='.'
      KDOT    = 3
      ISTART  = 2
      IF(IABS(IEXP).GE.10) GO TO 170
      ID=8                                 ! 1 Digit Exponent
      INMANT=(1.0D+05)*ZN
      IF(INMANT.lt.INMAN6) go to 180
      INMANT=INMAN5
      IEXP=IEXP+1
      IF(IABS(IEXP).LT.10) GO TO 180
  170 ID=7                                 ! 2 Digit Exponent
      INMANT=(1.0D+04)*ZN
      IF(INMANT.lt.INMAN5) go to 180
      INMANT=INMAN4
      IEXP=IEXP+1
C
C     DEFINE MANTISSA
C
  180 IEXPS=ID+1
      II=ID
      DO 190 I=ISTART,ID
      IF(II.EQ.KDOT) GO TO 190
      I2=INMANT/10
      I3=INMANT-10*I2
      FIELD(II)=DIGITS(I3)
      INMANT=I2
  190 II=II-1
C
C     E
C
      FIELD(IEXPS) = 'E'
      IEXPS = IEXPS + 1
C
C     SIGN OF EXPONENT
C
      IF(IEXP.ge.0) go to 200
      IEXP=-IEXP
      FIELD(IEXPS)='-'
      GO TO 210
  200 FIELD(IEXPS)='+'
C
C     EXPONENT
C
  210 IF(IEXP.lt.10) go to 220
      KEXP=IEXP/10
      FIELD(10)=DIGITS(KEXP)
      IEXP=MOD(IEXP,10)
  220 FIELD(11)=DIGITS(IEXP)
c
c     If using column 1 but mantissa ends in 0, move mantissa right.
c
      if(KDOT.eq.2.and.FIELD(ID).eq.'0') then
      do k=ID-1,1,-1
      FIELD(k+1)=FIELD(k)
      enddo
      FIELD(1) = ' '
      endif
      RETURN
      END
      SUBROUTINE IN9(E,FIELD)
C=======================================================================
C
C     PURPOSE
C     =======
C     CONVERT FROM HOLLERITH TO FLOATING POINT.
C     12/18/2012 - EXTENDED FOR 3 DIGIT EXPONENT (I Thank Viktor Zerkin)
C                  More than 3 digit exponent = ERROR - return 0.0
C
C     ARGUMENTS
C     =========
C     E       = FLOATING POINT NUMBER (OUTPUT)
C     FIELD   = 11A1 CHARACTER STRING (INPUT)
C
C     METHOD
C     ======
C     FIELD IS A STRING OF 11 CHARACTERS.
C     IT IS CONVERTED INTO A FLOATING POINR NUMBER (E)
C
C=======================================================================
      implicit real*8 (a-h,o-z)
      implicit integer*4 (i-n)
      save
      CHARACTER*1 MESS,DIGIT,FIELD,IFIELD
C-----12/18/2012 - EXTENDED FOR 3 DIGIT EXPONENTS
      DIMENSION FIELD(11),TENS(-99:99),DIGIT(0:9),MESS(11),XDIG(0:9)
      DATA MESS/' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' '/
      DATA DIGIT/'0','1','2','3','4','5','6','7','8','9'/
      DATA XDIG/0.0D+00,1.0D+00,2.0D+00,3.0D+00,4.0D+00,
     1          5.0D+00,6.0D+00,7.0D+00,8.0D+00,9.0D+00/
C-----ON FIRST CALL DEFINE POWERS OF 10
      DATA IPASS/0/
      IF(IPASS.NE.0) GO TO 20
      IPASS=1
      TENS(0)=1.0D+00
C-----12/18/2012 - EXTENDED FOR 3 DIGIT EXPONENTS
      DO 10 I=1,99
      TENS( I)=TENS(I-1)*10.0D+00
   10 TENS(-I)=TENS(1-I)/10.0D+00
C
C     TRANSLATE MANTISSA.
C
C-----SKIP LEADING BLANK CHARACTERS.
   20 DO 30 I=1,11
      IF(FIELD(I).NE.' ') GO TO 40
   30 CONTINUE
C-----FIELD IS COMPLETELY BLANK. RETURN ZERO.
      E=0.0D+00
      RETURN
C-----INITIALIZE SIGN TO PLUS AND THEN CHECK FOR LEADING MINUS SIGN.
   40 SIGN=1.0D+00
C-----06/02/09 - ADDED TEST FOR LEADING + SIGN.
      IF(FIELD(I).NE.'-') GO TO 50
      SIGN=-1.0D+00
      I=I+1
c-----added leading +
      GO TO 60
   50 IF(FIELD(I).EQ.'+') I = I + 1
C-----INITIALIZE FIXED POINT INPUT FIELD AND POSITION OF DECIMAL POINT.
   60 X9IN=0.0
      IPT=-20
      IMZERO=0
C-----SCAN REMAINDER OF MANTISSA.
      DO 120 J=I,11
      IFIELD=FIELD(J)
C-----SCAN FOR DIGIT OR DECIMAL POINT (WHICH ARE PART OF MANTISSA).
      DO 70 K=0,9
      IF(IFIELD.EQ.DIGIT(K)) GO TO 90
   70 CONTINUE
      IF(IFIELD.NE.'.') GO TO 80
      IPT=0
      GO TO 120
C-----SCAN FOR BLANK (WHICH ENDS MANTISSA).
   80 IF(IFIELD.EQ.' ') GO TO 130
C-----SCAN FOR e, E, d, D, - OR + (WHICH BEGINS EXPONENT).
      IF(IFIELD.EQ.'e'.OR.IFIELD.EQ.'E') GO TO 160
      IF(IFIELD.EQ.'d'.OR.IFIELD.EQ.'D') GO TO 160
      IF(IFIELD.EQ.'-') GO TO 190
      IF(IFIELD.EQ.'+') GO TO 170
C-----ERROR. CANNOT IDENTIFY CHARACTER.
      GO TO 240
C-----DIGIT FOUND. SAVE TRAILING ZEROES AFTER DECIMAL POINT.
   90 IF(IPT.LT.0) GO TO 110
      IF(K.NE.0) GO TO 100
C-----SAVE TRAILING ZEROES.
      IMZERO=IMZERO+1
      GO TO 120
  100 IF(IMZERO.LE.0) GO TO 110
C-----INSERT ZEROES BEFORE NEXT NUMBER.
      X9IN=10.0D+00*X9IN
      IPT=IPT+1
      IMZERO=IMZERO-1
      GO TO 100
C-----DIGIT FOUND. INCREMENT FIXED POINT EQUIVALENT AND DECIMAL POINT
C-----OFFSET.
  110 X9IN=10.0D+00*X9IN+XDIG(K)
      IPT=IPT+1
  120 CONTINUE
C-----ENTIRE FIELD TRANSLATED (NO EXPONENT). CONVERT TO FLOATING POINT.
      GO TO 150
C-----BLANK FOUND (END OF MANTISSA). SCAN REMAINDER OF FIELD FOR
C-----EXPONENT.
  130 I=J+1
      IF(I.GT.11) GO TO 150
      DO 140 J=I,11
      IFIELD=FIELD(J)
      IF(IFIELD.EQ.' ') GO TO 140
      IF(IFIELD.EQ.'e'.OR.IFIELD.EQ.'E') GO TO 160
      IF(IFIELD.EQ.'d'.OR.IFIELD.EQ.'D') GO TO 160
      IF(IFIELD.EQ.'-') GO TO 190
      IF(IFIELD.EQ.'+') GO TO 170
C-----ERROR. CANNOT IDENTIFY CHARACTER.
      GO TO 240
  140 CONTINUE
C-----ENTIRE FIELD TRANSLATED (NO EXPONENT). CONVERT TO FLOATING POINT.
  150 E=X9IN
      IF(IPT.GT.0) E=E/TENS(IPT)
      E=SIGN*E
      RETURN
C
C     TRANSLATE EXPONENT.
C
C-----BEGINNING OF EXPONENT FOUND (E OR D). CHECK FOR FOLLOWING - OR +.
  160 J=J+1
      IFIELD=FIELD(J)
      IF(IFIELD.EQ.'-') GO TO 190
      IF(IFIELD.NE.'+') GO TO 180
C----- + FOUND. INITIALIZE EXPONENT SIGN.
  170 J=J+1
  180 ISIGN=1
      GO TO 200
C----- - FOUND. INITIALIZE EXPONENT SIGN.
  190 J=J+1
      ISIGN=-1
C-----INITIALIZE EXPONENT AND SCAN REMAINING CHARACTERS FOR EXPONENT.
  200 IEXP=0
      DO 230 I=J,11
      IFIELD=FIELD(I)
      IF(IFIELD.EQ.' ') GO TO 230
      DO 210 K=0,9
      IF(IFIELD.EQ.DIGIT(K)) GO TO 220
  210 CONTINUE
C-----ERROR. CANNOT IDENTIFY CHARACTER.
      GO TO 240
C-----DIGIT FOUND. INCREMENT EXPONENT.
C-----OFFSET.
  220 IEXP=10*IEXP+K
  230 CONTINUE
C-----ENTIRE FIELD TRANSLATED (WITH EXPONENT). CONVERT TO FLOATING
C-----POINT.
      E=X9IN
      IEXP=ISIGN*IEXP
      IF(IPT.GT.0) IEXP=IEXP-IPT
C-----12/18/2012 - EXTENDED FOR 3 DIGIT EXPONENTS
C-----11/22/2013 - Restricted to 2 Digit Eexponent
      IF(iabs(IEXP).GT.99) GO TO 240
      E=SIGN*E*TENS(IEXP)
      RETURN
C
C     ERROR CONDITIONS.
C
C-----ILLEGAL CHARACTER.
  240 MESS(J)='*'
      write(*,250) FIELD,MESS
  250 FORMAT(1X,11A1/1X,11A1/' ERROR in Input Data...Translated as 0.')
      E=0.0D+00
      MESS(J)=' '
      RETURN
      END
      SUBROUTINE ENDIT
C=======================================================================
C
C     NORMAL END - Print code name but NO TIME.
C
C=======================================================================
      IMPLICIT REAL*4 (A-H,O-Z)  ! note - REAL*4 not 8
      IMPLICIT INTEGER*4 (I-N)
      SAVE
      CHARACTER*8 CODENAME
      COMMON/NAMECODE/CODENAME
      WRITE(3,10) CODENAME
      WRITE(*,10) CODENAME
      STOP
C=======================================================================
C
C     ERROR END - Print ERROR message but NO TIME.
C
C=======================================================================
      ENTRY ENDERROR
      WRITE(3,20)
      WRITE(*,20)
   10 FORMAT(1X,71('=')/1X,A8/1X,71('='))
   20 FORMAT(1X,71('=')/1X,'   ERROR'/1X,71('='))
      STOP
      END
